      *****************************************************************
      *               ASCEBCB4.CPY - a COBOL Copy File                *
      *         Copyright (C) 1987-2018 SimoTime Technologies         *
      *                     All Rights Reserved                       *
      *              Provided by SimoTime Technologies                *
      *        Our e-mail address is: helpdesk@simotime.com           *
      *     Also, visit our Web Site at http://www.simotime.com       *
      *****************************************************************
      * The following tables are used by the INSPECT statement to do  *
      * the conversion between EBCDIC and ASCII.                      *
      *   inspect FIELD-NAME converting EBCDIC-INFO to ASCII-INFO     *
      *   inspect FIELD-NAME converting ASCII-INFO  to EBCDIC-INFO    *
      *                                                               *
      * The tables support the US Characters set and not the          *
      * international character set.                                  *
      *                                                               *
      * The tables may also be used to convert between lower and      *
      * upper case.                                                   *
      *   inspect FIELD-NAME converting EBCDIC-LOWER to EBCDIC-UPPER  *
      *   inspect FIELD-NAME converting ASCII-LOWER  to ASCII-UPPER   *
      *                                                               *
      * SimoZAPS contains four tables that may be used for various    *
      * Upper/Lower Case or EBCDIC/ASCII conversion requirements.     *
      * ASCEBCB1.CPY - includes a full character set for the alphabet *
      *                (upper/lower case), digit, special characters  *
      *                and alternate codes for characters with the    *
      *                acute, grave, umlaut, tilde and circumflex.    *
      *                Caution: this table does not convert the Y     *
      *                characters with an umlat.                      *
      * ASCEBCB2.CPY - includes the character set for the translation *
      *                between EBCDIC/ASCII of signed/unsigned,       *
      *                zoned-decimal, numeric fields.                 *
      * ASCEBCB3.CPY - includes the character set for the alternate   *
      *                codes with the acute, grave, umlaut, tilde and *
      *                circumflex. This is primarily used for case    *
      *                conversion.                                    *
      *                Note: this table converts the Y characters     *
      *                with an umlat, this may convert high-values    *
      *                X'FF' to X'DF'                                 *
      * ASCEBCB4.CPY - includes the character set for the alphabet    *
      *                (upper/lower case), digit, special characters. *
      *                This is primarily used in the US where the     *
      *                alternate codes may not be required.           *
      * ASCEBCB5.CPY - includes a full character set for the alphabet *
      *                (upper/lower case), digit, special characters  *
      *                and alternate codes for characters with the    *
      *                acute, grave, umlaut, tilde and circumflex.    *
      *                Caution: this table will convert the Y         *
      *                characters with an umlat.                      *
      *****************************************************************
      *
      *    ------------------------------------------------------------
      *    **  The EBCDIC Table ...
      *    **  01  EBCDIC space character     40
      *    **  02                             A B C D E F G H I
      *    **  03                             J K L M N O P Q R
      *    **  04                             S T U V W X Y Z
      *    **  05                             a b c d e f g h i
      *    **  06                             j k l m n o p q r
      *    **  07                             s t u v w x y z
      *    **  08                             0 1 2 3 4 5 6 7 8 9
      *    **  09                             . < ( + | & ! $ * )
      *    **  10                             ;5F - / , % _ > ? `
      *    **  11  7D/7F Single/Double quote  : # @7D =7F [ ] { }
      *    **  12                             \ ~ ^
       01  EBCDIC-DATA.
           05  EBCDIC-SPACE pic X     value X'40'.
           05  EBCDIC-UPPER-CASE-DATA.
               10  filler   pic X(9)  value X'C1C2C3C4C5C6C7C8C9'.      A-I
               10  filler   pic X(9)  value X'D1D2D3D4D5D6D7D8D9'.      J-R
               10  filler   pic X(8)  value X'E2E3E4E5E6E7E8E9'.        S-Z
           05  EBCDIC-UPPER redefines EBCDIC-UPPER-CASE-DATA
                            pic X(26).
           05  EBCDIC-LOWER-CASE-DATA.
               10  filler   pic X(9)  value X'818283848586878889'.      a-i
               10  filler   pic X(9)  value X'919293949596979899'.      j-r
               10  filler   pic X(8)  value X'A2A3A4A5A6A7A8A9'.        s-z
           05  EBCDIC-LOWER redefines EBCDIC-LOWER-CASE-DATA
                            pic X(26).
           05  EBCDIC-DIGITS.
               10  filler   pic X(10) value X'F0F1F2F3F4F5F6F7F8F9'.    0-9
           05  EBCDIC-SPECIAL.
               10  filler   pic X(8)  value X'4B4C4D4E4F505A5B'.
      *                                        . < ( + | & ! $
               10  filler   pic X(8)  value X'5C5D5E5F60616B6C'.
      *                                        * ) ;  - / , %
               10  filler   pic X(8)  value X'6D6E6F797A7B7C7D'.
      *                                        _ > ? ` : # @ '
               10  filler   pic X(8)  value X'7E7FBABBC0D0E0A1'.
      *                                        = " [ ] { } \ ~
               10  filler   pic X(6)  value X'4A6AB020B1B2'.
      *                                          ^   
      *    **  EBCDIC-SPECIAL...
      *    **  X'4A' is US Cent Sign, X'6A' is two-piece vertical bar
      *    **  X'20' is Euro, X'B1' is British Pound, X'B2' is Yen
       01  E-INFO           redefines EBCDIC-DATA pic X(101).
       01  EBCDIC-TABLE     redefines EBCDIC-DATA.
           05  EBCDIC-BYTE  pic X     occurs 101 times.
      *
      *    ------------------------------------------------------------
      *    **  The ASCII Table ...
      *    **  01  ASCII space character      20
      *    **  02                             A B C D E F G H I
      *    **  03                             J K L M N O P Q R
      *    **  04                             S T U V W X Y Z
      *    **  05                             a b c d e f g h i
      *    **  06                             j k l m n o p q r
      *    **  07                             s t u v w x y z
      *    **  08                             0 1 2 3 4 5 6 7 8 9
      *    **  09                             . < ( + | & ! $ * )
      *    **  10                             ;5F - / , % _ > ? `
      *    **  11  27/22 Single/Double quote  : # @27 =22 [ ] { }
      *    **  12                             \ ~ ^
       01  ASCII-DATA.
           05  ASCII-SPACE  pic X     value X'20'.
           05  ASCII-UPPER-CASE-DATA.
               10  filler   pic X(9)  value X'414243444546474849'.      A-I
               10  filler   pic X(9)  value X'4A4B4C4D4E4F505152'.      J-R
               10  filler   pic X(8)  value X'535455565758595A'.        S-Z
           05  ASCII-UPPER  redefines ASCII-UPPER-CASE-DATA
                            pic X(26).
           05  ASCII-LOWER-CASE-DATA.
               10  filler   pic X(9)  value X'616263646566676869'.      a-i
               10  filler   pic X(9)  value X'6A6B6C6D6E6F707172'.      j-r
               10  filler   pic X(8)  value X'737475767778797A'.        s-z
           05  ASCII-LOWER  redefines ASCII-LOWER-CASE-DATA
                            pic X(26).
           05  ASCII-DIGITS.
               10  filler   pic X(10) value X'30313233343536373839'.    0-9
           05  ASCII-SPECIAL.
               10  filler   pic X(8)  value X'2E3C282B7C262124'.
      *                                        . < ( + | & ! $
               10  filler   pic X(8)  value X'2A293BAC2D2F2C25'.
      *                                        * ) ;  - / , %
               10  filler   pic X(8)  value X'5F3E3F603A234027'.
      *                                        _ > ? ` : # @ '
               10  filler   pic X(8)  value X'3D225B5D7B7D5C7E'.
      *                                        = " [ ] { } \ ~
               10  filler   pic X(6)  value X'A2A65E80A3A5'.
      *                                          ^   
      *    **  ASCII-SPECIAL...
      *    **  X'A2' is US Cent Sign, X'A6' is two-piece vertical bar
      *    **  X'80' is Euro, X'A3' is British Pound, X'A5' is Yen
       01  A-INFO           redefines ASCII-DATA pic X(101).
       01  ASCII-TABLE      redefines ASCII-DATA.
           05  ASCII-BYTE   pic X     occurs 101 times.
      *
      ***  ASCEBCB4 - End-of-Copy File - - - - - - - - - - - ASCEBCB4 *
      *****************************************************************
      *
