      *****************************************************************
      *     Data Structure or Pass Area used for calling SIMODATE.    *
      *****************************************************************
      *         Copyright (C) 1987-2019 SimoTime Technologies         *
      *                     All Rights Reserved                       *
      *****************************************************************
      *              Provided by SimoTime Technologies                *
      *        Our e-mail address is: helpdesk@simotime.com           *
      *     Also, visit our Web Site at http://www.simotime.com       *
      *****************************************************************
      * STD-REQUEST          - This is an eight character data string
      *                        and should contain the following in
      *                        upper case.
      *                        EDITDATE, This keyword causes the date
      *                        routine (SIMODATE) to validate the date
      *                        and provide an edited text string plus
      *                        a julian date and more.
      *                        EDITJDAT, WIP
      *                        The calling program must provide the
      *                        above as a request. Otherwise, a non-
      *                        zero return code will be set in the
      *                        following result field. This data string
      *                        is not changed by the date routine.
      *
      * STD-RESPONSE         - This is a four byte binary data string
      *                        and is used by the conversion routine to
      *                        pass a return code. If the date request
      *                        is successful the value in this data
      *                        string will be zero.
      *
      * STD-MESSAGE-TEXT     - If a non-zero response occurs this field
      *                        will contain a short message.
      *
      * STD-GREGORIAN-4-EDIT - This is the input date field in
      *                        gregorian format that will be validated.
      *                        If the gregorian date does not pass the
      *                        editing process a message will be posted
      *                        in the STD-MESSAGE-TEXT field and a
      *                        non-zero value of 16 will be placed in
      *                        the STD-RESPONSE field. SimoDATE is
      *                        somewhat flexible in accepting and
      *                        reformatting a gregorian date. For
      *                        example, the following are acceptable
      *                        and will be reformatted into the
      *                        STD-GREGORIAN-4-EDIT field.
      *                        ccyymmdd   This is the preferred format.
      *                        ccyy-mm-dd Special characters may be
      *                                   used as separators.
      *                        ccyy-m-d   Single digit month and day
      *                                   values are acceptable when
      *                                   delimiter are used.
      *                        ccyy/m/dd  A mixing of single digit
      *                                   usage is accepted.
      *
      * STD-DEBUG-INFO       - This data string provides additional
      *                        debugging capability. This may be
      *                        helpful if changes are made to the date
      *                        routine.
      *
      * STD-LEAP-YEAR-YN     - This field is used to identify a leap
      *                        year. A "Y" indicates the input date is
      *                        a leap year. An "N" indicates the input
      *                        year is not a leap year.
      *
      * STD-MONTH-VERBAGE    - This field is used for the text of the
      *                        input month. For example, 1=January,
      *                        2=February, etc.
      *
      * STD-MM-DAYS          - This field is used for the number of
      *                        days in the input month.
      *
      * STD-GREGORIAN-DATE   - This field is used for the gregorian
      *                        date in ccyymmdd format (all numeric
      *                        values).
      *
      * STD-JULIAN-DATE      - This field is used for the julian date
      *                        in ccyyddd format.
      *
      * STD-DAYS-REMAINING   - The number of days remaining in the
      *                        year.
      *
      * STD-DATE-VERBAGE     - This field is used for the date in text
      *                        format. For example, 19991225 will
      *                        produce a text string of
      *                        December 25, 1999.
      *
      * STD-DATE-EDIT-BYTE   - This field contains the character that
      *                        will be used to edit and format the
      *                        gregorian date. For example, this field
      *                        may contain a space, a dash, a slash,
      *                        etc... and will be used as the delimiter
      *                        between the year, month and day.
      *
      * STD-DATE-EDITED      - This field if for the edited date using
      *                        the delimiter character from the
      *                        preceding field. For example, 19991225
      *                        will produce 1999/12/25 if the delimiter
      *                        character is a slash.
      * STD-DAY-OF-WEEK      - WIP, 1-7 is Sunday through Saturday.
      *****************************************************************
      *
       01  STD-SIMODATE.
           05  STD-REQUEST             pic X(8).
           05  STD-RESPONSE            pic 9(4).
           05  STD-MESSAGE-TEXT        pic X(68).
           05  STD-GREGORIAN-4-EDIT    pic X(10).
           05  STD-EDITED-INFO.
               10  STD-DEBUG-INFO      pic X(8).
               10  STD-LEAP-YEAR-YN    pic X.
               10  STD-MONTH-VERBAGE   pic X(10).
               10  STD-MM-DAYS         pic 99.
               10  STD-GREGORIAN-DATE  pic 9(8).
               10  STD-JULIAN-DATE     pic 9(7).
               10  STD-JULIAN-VALUE    redefines STD-JULIAN-DATE.
                   15  STD-JULIAN-CCYY pic 9(4).
                   15  STD-JULIAN-DAY  pic 9(3).
               10  STD-DAYS-REMAINING  pic 9(3).
               10  STD-DATE-VERBAGE    pic X(18).
               10  STD-DATE-EDIT-BYTE  pic X.
               10  STD-DATE-EDITED-14.
                   15  STD-DATE-DAY    pic X(3).
                   15  STD-DATE-DAY-2  pic X.
                   15  STD-DATE-EDITED pic X(10).
           05  STD-DAY-OF-WEEK.
               10  STD-DAY-OF-WEEK-NBR pic 9.
               10  STD-DAY-OF-WEEK-TXT pic X(10).
      ***  PASSDATE - End-of-Copy File - - - - - - - - - - - PASSDATE *
      *****************************************************************
      *
