       IDENTIFICATION DIVISION.
       PROGRAM-ID.    SIMOCASE.
       AUTHOR.        SIMOTIME TECHNOLOGIES.
      *****************************************************************
      * Copyright (C) 1987-2019 SimoTime Technologies.                *
      *                                                               *
      * All rights reserved.  Unpublished, all rights reserved under  *
      * copyright law and international treaty.  Use of a copyright   *
      * notice is precautionary only and does not imply publication   *
      * or disclosure.                                                *
      *                                                               *
      * Permission to use, copy, modify and distribute this software  *
      * for any commercial purpose requires a fee to be paid to       *
      * SimoTime Technologies. Once the fee is received by SimoTime   *
      * the latest version of the software will be delivered and a    *
      * license will be granted for use within an enterprise,         *
      * provided the SimoTime copyright notice appear on all copies   *
      * of the software. The SimoTime name or Logo may not be used    *
      * in any advertising or publicity pertaining to the use of the  *
      * software without the written permission of SimoTime           *
      * Technologies.                                                 *
      *                                                               *
      * Permission to use, copy and modify this software for any      *
      * non-commercial purpose and without fee is hereby granted,     *
      * provided the SimoTime copyright notice appear on all copies   *
      * of the software. The SimoTime name or Logo may not be used    *
      * in any advertising or publicity pertaining to the use of the  *
      * software without the written permission of SimoTime           *
      * Technologies.                                                 *
      *                                                               *
      * SimoTime Technologies makes no warranty or representations    *
      * about the suitability of the software for any purpose. It is  *
      * provided "AS IS" without any expressed or implied warranty,   *
      * including the implied warranties of merchantability, fitness  *
      * for a particular purpose and non-infringement. SimoTime       *
      * Technologies shall not be liable for any direct, indirect,    *
      * special or consequential damages resulting from the loss of   *
      * use, data or projects, whether in an action of contract or    *
      * tort, arising out of or in connection with the use or         *
      * performance of this software                                  *
      *                                                               *
      * SimoTime Technologies                                         *
      * 15 Carnoustie Drive                                           *
      * Novato, CA 94949-5849                                         *
      * 415.883.6565                                                  *
      *                                                               *
      * RESTRICTED RIGHTS LEGEND                                      *
      * Use, duplication, or disclosure by the Government is subject  *
      * to restrictions as set forth in subparagraph (c)(1)(ii) of    *
      * the Rights in Technical Data and Computer Software clause at  *
      * DFARS 52.227-7013 or subparagraphs (c)(1) and (2) of          *
      * Commercial  Computer Software - Restricted Rights  at 48      *
      * CFR 52.227-19, as applicable.  Contact SimoTime Technologies, *
      * 15 Carnoustie Drive, Novato, CA 94949-5849.                   *
      *                                                               *
      *****************************************************************
      *      This program is provided by SimoTime Technologies        *
      *        Our e-mail address is: helpdesk@simotime.com           *
      *     Also, visit our Web Site at http://www.simotime.com       *
      *                                                               *
      *****************************************************************
      *
      *****************************************************************
      * Source Member: SIMOCASE.CBL
      * Copy Files:    PASSCASE.CPY
      *****************************************************************
      *
      * SIMOCASE - A called routine for case conversions.
      *
      * CALLING PROTOCOL
      * ----------------
      * Use standard COBOL calling procedures.
      *
      * DESCRIPTION
      * -----------
      * This program will do alphabetic case conversions.
      *
      * When CASE-PASS-REQUEST
      *      is '0' Do Upper to Lower case conversion
      *      is '1' Do Lower to Upper case conversion
      *      is '2' Do capitalization of each word
      *      is '3' Do Edit key words from a table
      *
      ****************************************************************
      *
      * MAINTENANCE
      * -----------
      * 1989/02/27 Simmons, Created program.
      * 1989/02/27 Simmons, no changes to date
      *
      *****************************************************************
      *
       DATA DIVISION.
       WORKING-STORAGE SECTION.
      *
      *****************************************************************
      *    Data-structure for Title and Copyright...
      *    ------------------------------------------------------------
       01  SIM-TITLE.
           05  T1 pic X(11) value '* SIMOCASE '.
           05  T2 pic X(34) value 'Callable Case Conversion Routine  '.
           05  T3 pic X(10) value ' v08.06.20'.
           05  T4 pic X(24) value ' http://www.simotime.com'.
       01  SIM-COPYRIGHT.
           05  C1 pic X(11) value '* SIMOCASE '.
           05  C2 pic X(20) value 'Copyright 1987-2019 '.
           05  C3 pic X(28) value '   SimoTime Technologies    '.
           05  C4 pic X(20) value ' All Rights Reserved'.

       01  FIRST-TIME              pic X       value 'Y'.

       01  MESSAGE-BUFFER.
           05  MESSAGE-HEADER      pic X(11)   value '* SIMOCASE '.
           05  MESSAGE-TEXT        pic X(68).

       01  TWO-BYTES.
           05  TWO-BYTES-01        pic X.
           05  TWO-BYTES-02        pic X.
       01  TWO-BYTES-BINARY        redefines   TWO-BYTES
                                   pic S9(4)   binary.

       01  IX-1                    pic 9(3)    value 0.
       01  SZ-1                    pic 9(3)    value 0.
       01  STRING-SIZE             pic 9(3)    value 0.
       01  WORK-12A                pic X(12)   value SPACES.
       01  WORK-12B                pic X(12)   value SPACES.

       COPY CASEBK01.
       COPY CASEBK02.
       COPY CASEBK03.

      *****************************************************************
       LINKAGE SECTION.
       COPY PASSCASE.

      *****************************************************************
       PROCEDURE DIVISION using CASE-PASS-AREA.

       MAIN-ROUTINE.
           move ZERO to CASE-PASS-RESULT
           if  CASE-PASS-STRING-SIZE is numeric
           and CASE-PASS-STRING-SIZE > 0
           and CASE-PASS-STRING-SIZE < 257
               add CASE-PASS-STRING-SIZE to ZERO giving STRING-SIZE
           else
               perform INVALID-STRING-SIZE
           end-if

           if  FIRST-TIME not = 'N'
               perform Z-POST-COPYRIGHT
               move 'N' to FIRST-TIME
           end-if

           evaluate CASE-PASS-REQUEST
               when '0'      perform CASE-CONVERT-00
               when '1'      perform CASE-CONVERT-01
               when '2'      perform CASE-CONVERT-02
               when '3'      perform CASE-CONVERT-03
               when OTHER    perform INVALID-REQUEST
           end-evaluate

           add CASE-PASS-RESULT to ZERO giving RETURN-CODE

           GOBACK.

      *****************************************************************
       INVALID-REQUEST.
           display '* SIMOCASE, INVALID REQUEST, ' CASE-PASS-REQUEST
           add 16 to ZERO giving CASE-PASS-RESULT
           add CASE-PASS-RESULT to ZERO giving RETURN-CODE
           GOBACK.
      *    exit.

      *****************************************************************
       INVALID-STRING-SIZE.
           display '* SIMOCASE, INVALID STRING SIZE.'
           add 16 to ZERO giving CASE-PASS-RESULT
           add CASE-PASS-RESULT to ZERO giving RETURN-CODE
           GOBACK.
      *    exit.

      *****************************************************************
       CASE-CONVERT-00.
      *    Convert a text string to lower case
           inspect CASE-PASS-BUFFER(1:STRING-SIZE)
           converting U-CASE to L-CASE
           exit.

      *****************************************************************
       CASE-CONVERT-01.
      *    Convert a text string to UPPER case
           inspect CASE-PASS-BUFFER(1:STRING-SIZE)
           converting L-CASE to U-CASE
           exit.

      *****************************************************************
       CASE-CONVERT-02.
      *    Capitalize each word in a text string
           perform varying IX-1 from 1 by 1 until IX-1 > 26
             inspect CASE-PASS-BUFFER(1:STRING-SIZE)
             replacing all CAPS-LOWER-TABLE(IX-1)
             by CAPS-UPPER-TABLE(IX-1)
           end-perform
           inspect CASE-PASS-BUFFER(1:1)
           converting L-CASE to U-CASE
           exit.

      *****************************************************************
       CASE-CONVERT-03.
      *    Edit specific words
           perform varying IX-1 from 1 by 1
           until IX-1 > CAPS-TABLE-LIMIT
             move CAPS-TABLE-OLD(IX-1) to WORK-12A
             move CAPS-TABLE-NEW(IX-1) to WORK-12B
             add  CAPS-TABLE-SIZE(IX-1) to ZERO giving SZ-1
             inspect CASE-PASS-BUFFER(1:STRING-SIZE)
             replacing all WORK-12A(1:SZ-1)
             by WORK-12B(1:SZ-1)
           end-perform
           exit.

      *****************************************************************
      *    Display the Copyright data-structure...
      *    ------------------------------------------------------------
       Z-POST-COPYRIGHT.
           display SIM-TITLE
           display SIM-COPYRIGHT
           exit.
       Z-POST-MESSAGE.
           display MESSAGE-BUFFER
           move SPACES to MESSAGE-TEXT
           exit.
      *****************************************************************
      *      This example is provided by SimoTime Technologies        *
      *        Our e-mail address is: helpdesk@simotime.com           *
      *     Also, visit our Web Site at http://www.simotime.com       *
      *****************************************************************
